// Copyright 2020 Espressif Systems (Shanghai) PTE LTD
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at

//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#ifndef __SNTP_H__
#define __SNTP_H__

/*
 * This header is provided only for compatibility reasons for existing
 * applications which directly include "sntp.h" from the IDF default paths.
 * and will be removed in IDF v5.0.
 * It is recommended to use "esp_sntp.h" from IDF's lwip port folder
 */

#include "esp_sntp.h"

#endif // __SNTP_H__
