// Copyright 2010-2020 Espressif Systems (Shanghai) PTE LTD
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at

//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#pragma once

//Touch channels
#define TOUCH_PAD_GPIO4_CHANNEL     TOUCH_PAD_NUM0
#define TOUCH_PAD_NUM0_GPIO_NUM     4

#define TOUCH_PAD_GPIO0_CHANNEL     TOUCH_PAD_NUM1
#define TOUCH_PAD_NUM1_GPIO_NUM     0

#define TOUCH_PAD_GPIO2_CHANNEL     TOUCH_PAD_NUM2
#define TOUCH_PAD_NUM2_GPIO_NUM     2

#define TOUCH_PAD_GPIO15_CHANNEL    TOUCH_PAD_NUM3
#define TOUCH_PAD_NUM3_GPIO_NUM     15

#define TOUCH_PAD_GPIO13_CHANNEL    TOUCH_PAD_NUM4
#define TOUCH_PAD_NUM4_GPIO_NUM     13

#define TOUCH_PAD_GPIO12_CHANNEL    TOUCH_PAD_NUM5
#define TOUCH_PAD_NUM5_GPIO_NUM     12

#define TOUCH_PAD_GPIO14_CHANNEL    TOUCH_PAD_NUM6
#define TOUCH_PAD_NUM6_GPIO_NUM     14

#define TOUCH_PAD_GPIO27_CHANNEL    TOUCH_PAD_NUM7
#define TOUCH_PAD_NUM7_GPIO_NUM     27

#define TOUCH_PAD_GPIO33_CHANNEL    TOUCH_PAD_NUM8
#define TOUCH_PAD_NUM8_GPIO_NUM     33

#define TOUCH_PAD_GPIO32_CHANNEL    TOUCH_PAD_NUM9
#define TOUCH_PAD_NUM9_GPIO_NUM     32
